<?php

/**
 * @file
 * Contains \Drupal\cforge\DemoUserData.
 */

namespace Drupal\cforge;
use Drupal\taxonomy\Entity\Term;
use Drupal\node\Entity\NodeType;

abstract class DemoUserData {

  static $firstNames = [];
  static $lastNames = [];
  static $names = [];
  static $streets = [];
  static $hoods = [];

  static function randNeighbourhood() {
    static $hoods = [];
    if (empty($hoods)) {
      $nodes = \Drupal::entityTypeManager()
        ->getStorage('node')
        ->loadByProperties(['type' => 'neighbourhood']);
      foreach ($nodes as $node) {
        $hoods[] = $node->getTitle();
      }
    }
    if (empty($hoods)) {
      drupal_set_message('no neighbourhoods created yet');
    }
    return $hoods[array_rand($hoods)];
  }

  static function randStreet() {
    $key = array_rand(static::$streets);
    return static::$streets[$key];
  }

  static function randPhone() {
    return '0'.rand(1000, 9999).' '.rand(100000,  999999);
  }

  static function createNeighbourhoods($hoods = []) {
    if (!$hoods) {
      $hoods = static::$hoods;
    }
    foreach ($hoods as $hood) {
      $exists = \Drupal::entityQuery('node')
        ->condition('type', 'neighbourhood')
        ->condition('title', $hood)
        ->execute();
      if(!$exists) {
        \Drupal\node\Entity\Node::create([
          'type' => 'neighbourhood',
          'title' => trim($hood),
          'uid' => 1
        ])->save();
      }
    }
  }



  //called from cforge_install, the final install task
  static function generateDemoData($install_state) {
    if(!isset($install_state['example'])) return;
    $batch = [];

    $batch['title'] = 'Generating example content';
    //create 26 users
    $flavour = cforge_get_flavour();
    $batch['operations'][] = [
      'generateContent',
      [
        $flavour.'_user',
        ['kill' => FALSE, 'num' => 26, 'pass' => 'password', 'roles' => ['trader'], 'time_range' => strtotime('-2 years')]
      ]
    ];
    $batch['operations'][] = [
      'generateContent',
      [
        'smallad',
        ['kill' => FALSE, 'num' => 120, 'type' => 'offer', 'since' => strtotime('-6 months')]
      ]
    ];
    $batch['operations'][] = [
      'generateContent',
      [
        'smallad',
        ['kill' => FALSE, 'num' => 40, 'type' => 'want', 'since' => strtotime('-6 months')]
      ]
    ];
    $batch['operations'][] = [
      'generateContent',
      [
        'mcapi_transaction',
        ['kill' => FALSE, 'num' => 200, 'type' => 'default']
      ]
    ];
    $batch['operations'][] = [
      'generateContent',
      [
        'mcapi_transaction',
        ['kill' => FALSE, 'num' => 40, 'type' => 'signature']
      ]
    ];
    $batch['operations'][] = [
      'generateContent',
      [
        'content',
        ['kill' => FALSE, 'num' => 15, 'node_types' => ['story' => TRUE], 'max_comments' => 6, 'title_length' => 8]
      ]
    ];
    if (NodeType::load('event')) {
      $batch['operations'][] = [
        'generateContent',
        [
          'content',
          ['kill' => FALSE, 'num' => 15, 'node_types' => ['event' => TRUE], 'max_comments' => 2, 'title_length' => 8]
        ]
      ];
    }
    if (NodeType::load('document')) {
      $batch['operations'][] = [
        'generateContent',
        [
          'content',
          ['kill' => FALSE, 'num' => 15, 'node_types' => ['document' => TRUE], 'max_comments' => 0, 'title_length' => 5]
        ]
      ];
    }
    if (NodeType::load('image')) {
      //create another 3 galleries
      $props = ['vid' => 'galleries', 'format' => 'plain_text'];
      Term::create($props + [
        'name' => t('Annnual street party'),
      ])->save();
      Term::create($props + [
        'name' => t('Gurning competition'),
      ])->save();
      Term::create($props + [
        'name' => t('Harvest'),
      ])->save();
      $batch['operations'][] = [
        'generateContent',
        [
          'content',
          ['kill' => FALSE, 'num' => 50, 'node_types' => ['image' => TRUE], 'max_comments' => 4, 'title_length' => 5]
        ]
      ];
    }
    $batch['finished'] = ['finishSetup'];
    $batch['file'] = 'profiles/cforge/modules/hamlets/cforge.install';
    return $batch;
  }

  /**
   * return a randomised firstname and last name
   */
  static function getName() {
    static $firstnames, $lastnames = [];
    if (!$firstnames) {
      $firstnames = static::$firstNames;
      shuffle($firstnames);
    }
    if (!$lastnames) {
      $lastnames = static::$lastNames;
      shuffle($lastnames);
    }
    return [array_pop($firstnames), array_pop($lastnames)];
  }

  static function countUsers() {
    return count(static::$firstNames);
  }

  static function vocab() {
    return [];
  }

}
